'use strict';

exports.__esModule = true;
exports.serialize = serialize;
exports.parse = parse;

var _level = require('./level');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _prando = require('prando');

var _prando2 = _interopRequireDefault(_prando);

var _potatoEngine = require('potato-engine');

var _stripBom = require('strip-bom');

var _stripBom2 = _interopRequireDefault(_stripBom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function serialize(campaign) {
  return ['' + campaign.levels.length, ...campaign.levels.map(_level.serialize)].join('\n');
}

function parse(text, seed) {
  const lines = (0, _stripBom2.default)(text).split(/\r?\n/);
  const nLevels = Number(lines[0]);
  (0, _invariant2.default)(!isNaN(nLevels), "A Kye Campaign's first line must be the number of levels in it.");
  let levelFirstLine = 1;
  const levels = [];
  for (let i = 0; i < nLevels; i++) {
    (0, _invariant2.default)(lines.length > levelFirstLine, '%s levels specified in campaign, but consumed all lines after reading %s levels.', nLevels, i);
    const levelSeed = seed && new _prando2.default(seed).nextInt(0, 99999999);
    const level = (0, _level.parse)(lines, levelFirstLine, i, levelSeed);
    levels.push(level);
    levelFirstLine += level.dimensions.height + _level.LEVEL_HEADER_LINES;
  }

  return new _potatoEngine.Campaign(levels, seed);
}