'use strict';

exports.__esModule = true;
exports.LEVEL_HEADER_LINES = undefined;
exports.parse = parse;
exports.serialize = serialize;

var _board = require('./board');

var _potatoEngine = require('potato-engine');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const LEVEL_HEADER_LINES = exports.LEVEL_HEADER_LINES = 3;
const FIRST_LAST_ROW = /[1-9]{3,}/;
const ROW = /[1-9].+[1-9]/;

function _parseHeader(headerLines) {
  (0, _invariant2.default)(headerLines.length === LEVEL_HEADER_LINES, 'Unexpected end of input');
  const [code, hint, completionMessage] = headerLines;
  (0, _invariant2.default)(code, 'A level must have a level code');
  return {
    code,
    hint,
    completionMessage
  };
}

function _findDimensions(lines, firstLine) {
  (0, _invariant2.default)(FIRST_LAST_ROW.test(lines[firstLine]), 'Expected board to start with a row containing only `5`s.');
  const width = lines[firstLine].length;
  let height = 1;
  let row;
  while ((row = lines[height + firstLine]) && ROW.test(row)) {
    (0, _invariant2.default)(row.length === width, 'Expected every line in the board to have the same length.');
    height++;
  }
  (0, _invariant2.default)(FIRST_LAST_ROW.test(lines[firstLine + height - 1]), 'Expected board to end with a row containing only `5`s.');
  return { width, height };
}

function parse(text, firstLine = 0, index = null) {
  const lines = typeof text === 'string' ? text.split(/\r?\n/) : text;
  const header = _parseHeader(lines.slice(firstLine, LEVEL_HEADER_LINES + firstLine));
  const dimensions = _findDimensions(lines, firstLine + LEVEL_HEADER_LINES);
  const board = (0, _board.parse)(lines.slice(firstLine + LEVEL_HEADER_LINES, firstLine + LEVEL_HEADER_LINES + dimensions.height));
  return new _potatoEngine.Level({ header, dimensions, board, index });
}

function serialize(level) {
  const { code, hint, completionMessage } = level.header;
  return [code, hint, completionMessage, (0, _board.serialize)(level.board)].join('\n');
}